(*
===============================================================================================

WinSock

===============================================================================================
WinSock Import Unit

Author:  Stefan Heymann
         Eschenweg 3
         72076 Tbingen
         Germany
         
URL:     www.destructor.de
E-Mail:  stefan@destructor.de

You can use this code according to the Destructor Source Licence (DSL)
===============================================================================================
Date        Who    What
-----------------------------------------------------------------------------------------------
2003-11-17  HeySt  Destructor-V2 Release
*)

UNIT WinSockInterface;

INTERFACE

USES
  Windows, SysUtils;

PROCEDURE LoadWinsockDll;
PROCEDURE UnloadWinsockDLL;
FUNCTION  WinSockDllLoaded : BOOLEAN;

FUNCTION  ErrorText (Error : INTEGER) : STRING;
FUNCTION  ErrorName (Error : INTEGER) : STRING;

(* ========================================================================================= *)


(* --- WINSOCK Types and Constants -------------------------------------------------------- *)

TYPE                     (* --- Base Types *)
  u_char  = CHAR;
  u_short = WORD;
  u_int   = CARDINAL;
  u_long  = LONGINT;
  p_long  = ^LONGINT;
  PPchar  = ^PChar;
  TSocket = u_int;

CONST
  INVALID_SOCKET     = TSocket (NOT (TSocket (0)));
  SOCKET_ERROR       = -1;
  INADDR_ANY         = u_long (0);
  INADDR_LOOPBACK    = $7F000001;
  INADDR_BROADCAST   = $FFFFFFFF;
  INADDR_NONE        = $FFFFFFFF;

  FD_SETSIZE         = 64;
  WSADESCRIPTION_LEN = 256;
  WSASYS_STATUS_LEN  = 128;


TYPE
  SunB         = PACKED RECORD
                   s_b1, s_b2, s_b3, s_b4 : u_char;
                 END;

  SunW         = PACKED RECORD
                   s_w1, s_w2 : u_short;
                 END;

  PIn_Addr     = ^TIn_Addr;
  TIn_Addr     = PACKED RECORD                        (* Internet Address *)
                   CASE INTEGER OF
                   0 : (S_un_b : SunB);
                   1 : (S_un_w : SunW);
                   2 : (S_addr : u_long);
                 END;
  PInAddr      = PIn_Addr;
  TInAddr      = TIn_Addr;

  pSockAddr_In = ^TSockAddr_In;            (* Internet Socket Address *)
  TSockAddr_In = PACKED RECORD
                   sin_family : u_short;
                   sin_port   : u_short;
                   sin_addr   : TIn_Addr;
                   sin_zero   : ARRAY [0..7] OF CHAR;
                 END;

  pSockAddr    = ^TSockAddr;
  TSockAddr    = PACKED RECORD                       (* Socket Address *)
                   sa_family : u_short;
                   sa_data   : ARRAY [0..13] OF CHAR;
                 END;

  pHostEnt     = ^THostEnt;
  THostEnt     = PACKED RECORD                    (* Host *)
                   h_name      : PChar;    (* Official name of the host (PC). *)
                   h_aliases   : PPChar;   (* A NULL-terminated array of alternate names. *)
                   h_addrtype  : integer;  (* The type of address being returned; for Windows Sockets this is always PF_INET. *)
                   h_length    : integer;  (* The length, in bytes, of each address; for PF_INET, this is always 4. *)
                   CASE INTEGER OF
                     0 : (h_addr_list : PPChar);
                     1 : (h_addr      : ^PInAddr);
                 END;

  pServEnt     = ^TServEnt;
  TServEnt     = PACKED RECORD                  (* --- Service *)
                   s_name    : PChar;    (* Official name of the service                                                  *)
                   s_aliases : PPChar;   (* A NULL-terminated array of alternate names                                    *)
                   s_port    : integer;  (* The port number (in network byte order) at which the service may be contacted *)
                   s_proto   : PChar;    (* The name of the protocol to use when contacting the service                   *)
                 END;

  pProtoEnt    = ^TProtoEnt;
  TProtoEnt    = PACKED RECORD                       (* Protocol *)
                   p_name    : PChar;
                   p_aliases : PPChar;
                   p_proto   : integer;
                 END;


  pFd_Set      = ^TFd_Set;
  TFd_Set      = PACKED RECORD
                   fd_count : u_short;
                   fd_array : ARRAY [0..FD_SETSIZE-1] OF TSocket;
                 END;

  TimeVal      = PACKED RECORD
                   tv_sec, tv_usec : LONGINT;
                 END;

  pWsaData     = ^TWsaData;
  TWsaData     = PACKED RECORD
                   wVersion       : word;
                   wHighVersion   : word;
                   szDescription  : ARRAY [0..WSADESCRIPTION_LEN] OF CHAR;
                   szSystemStatus : ARRAY [0..WSASYS_STATUS_LEN]  OF CHAR;
                   iMaxSockets    : u_short;
                   iMaxUdpDg      : u_short;
                   lpVendorInfo   : PChar;
                 END;

CONST
  TCP_NODELAY    = $0001;

  IPPROTO_IP     =     0;        (* TCP/IP protocols *)
  IPPROTO_ICMP   =     1;
  IPPROTO_GGP    =     2;
  IPPROTO_TCP    =     6;
  IPPROTO_PUP    =    12;
  IPPROTO_UDP    =    17;
  IPPROTO_IDP    =    22;
  IPPROTO_ND     =    77;

  IPPROTO_RAW    =   255;
  IPPROTO_MAX    =   256;


  IPPORT_ECHO        =   7;       (* TCP/IP well known ports *)
  IPPORT_DISCARD     =   9;
  IPPORT_SYSTAT      =  11;
  IPPORT_DAYTIME     =  13;
  IPPORT_NETSTAT     =  15;
  IPPORT_FTP         =  21;
  IPPORT_TELNET      =  23;
  IPPORT_SMTP        =  25;
  IPPORT_TIMESERVER  =  37;
  IPPORT_NAMESERVER  =  42;
  IPPORT_WHOIS       =  43;
  IPPORT_MTP         =  57;

  IPPORT_TFTP        =  69;
  IPPORT_RJE         =  77;
  IPPORT_FINGER      =  79;
  IPPORT_TTYLINK     =  87;
  IPPORT_SUPDUP      =  95;

  IPPORT_EXECSERVER  =  512;
  IPPORT_LOGINSERVER =  513;
  IPPORT_CMDSERVER   =  514;
  IPPORT_EFSSERVER   =  520;

  IPPORT_BIFFUDP     =  512;
  IPPORT_WHOSERVER   =  513;
  IPPORT_ROUTESERVER =  520;

  IPPORT_RESERVED    =  1024;

  IMPLINK_IP         =  155;
  IMPLINK_LOWEXPER   =  156;
  IMPLINK_HIGHEXPER  =  158;

  IP_OPTIONS     =   1;

  SOCK_STREAM    =   1;         (* Socket Types *)
  SOCK_DGRAM     =   2;
  SOCK_RAW       =   3;
  SOCK_RDM       =   4;
  SOCK_SEQPACKET =   5;

  SO_DEBUG       =   $0001;     (* Socket Options *)
  SO_ACCEPTCONN  =   $0002;
  SO_REUSEADDR   =   $0004;
  SO_KEEPALIVE   =   $0008;
  SO_DONTROUTE   =   $0010;
  SO_BROADCAST   =   $0020;
  SO_USELOOPBACK =   $0040;
  SO_LINGER      =   $0080;
  SO_OOBINLINE   =   $0100;

  SO_DONTLINGER  =   $ff7f;

  SO_SNDBUF      =   $1001;
  SO_RCVBUF      =   $1002;
  SO_SNDLOWAT    =   $1003;
  SO_RCVLOWAT    =   $1004;
  SO_SNDTIMEO    =   $1005;
  SO_RCVTIMEO    =   $1006;
  SO_ERROR       =   $1007;
  SO_TYPE        =   $1008;

  AF_UNSPEC      =   0;         (* Address Families *)
  AF_UNIX        =   1;
  AF_INET        =   2;
  AF_IMPLINK     =   3;
  AF_PUP         =   4;
  AF_CHAOS       =   5;
  AF_NS          =   6;
  AF_NBS         =   7;
  AF_ECMA        =   8;
  AF_DATAKIT     =   9;
  AF_CCITT       =   10;
  AF_SNA         =   11;
  AF_DECnet      =   12;
  AF_DLI         =   13;
  AF_LAT         =   14;
  AF_HYLINK      =   15;
  AF_APPLETALK   =   16;

  AF_MAX         =   17;

  PF_UNSPEC      =   AF_UNSPEC;    (* Protocol Families *)
  PF_UNIX        =   AF_UNIX;
  PF_INET        =   AF_INET;
  PF_IMPLINK     =   AF_IMPLINK;
  PF_PUP         =   AF_PUP;
  PF_CHAOS       =   AF_CHAOS;
  PF_NS          =   AF_NS;
  PF_NBS         =   AF_NBS;
  PF_ECMA        =   AF_ECMA;
  PF_DATAKIT     =   AF_DATAKIT;
  PF_CCITT       =   AF_CCITT;
  PF_SNA         =   AF_SNA;
  PF_DECnet      =   AF_DECnet;
  PF_DLI         =   AF_DLI;
  PF_LAT         =   AF_LAT;
  PF_HYLINK      =   AF_HYLINK;
  PF_APPLETALK   =   AF_APPLETALK;

  PF_MAX         =   AF_MAX;

  SOL_SOCKET     =   $FFFF;

  SOMAXCONN      =   5;

  MSG_OOB        =   $1;
  MSG_PEEK       =   $2;
  MSG_DONTROUTE  =   $4;

  MSG_MAXIOVLEN  =   16;

  MAXGETHOSTSTRUCT   =  1024;

  IOCPARM_MASK       = $7F;            (* parameters must be < 128 bytes *)
  IOC_VOID           = $20000000;      (* no parameters *)
  IOC_OUT            = $40000000;      (* copy out parameters *)
  IOC_IN             = $80000000;      (* copy in parameters *)
  IOC_INOUT          = IOC_IN OR IOC_OUT;   (* 0x20000000 distinguishes new & old ioctl's *)

  (*
  #define _IO(x,y)        (IOC_VOID OR (x SHL 8) OR y)
  #define _IOR(x,y,t)     (IOC_OUT OR ((sizeof(t) AND IOCPARM_MASK) SHL 16) OR (x SHL 8) OR y)
  #define _IOW(x,y,t)     (IOC_IN|(((long)sizeof(t)&IOCPARM_MASK)<<16)|(x<<8)|y)
  #define FIONREAD    _IOR('f', 127, u_long) /* get # bytes to read */
  #define FIONBIO     _IOW('f', 126, u_long) /* set/clear non-blocking i/o */
  #define FIOASYNC    _IOW('f', 125, u_long) /* set/clear async i/o */
  #define SIOCATMARK  _IOR('s',  7, u_long)  /* at oob mark? */
  *)

  FIONREAD           = (IOC_OUT OR ((LONGINT (SizeOf(LONGINT) AND IOCPARM_MASK)) SHL 16) OR (ORD ('f') SHL 8) OR 127);
  SIOCATMARK         = (IOC_OUT OR ((LONGINT (SizeOf(LONGINT) AND IOCPARM_MASK)) SHL 16) OR (ORD ('s') SHL 8) OR 7);

  FD_READ            =  $01;
  FD_WRITE           =  $02;
  FD_OOB             =  $04;
  FD_ACCEPT          =  $08;
  FD_CONNECT         =  $10;
  FD_CLOSE           =  $20;


(* --- Error Codes ----------------------------------------------------------------------- *)
CONST
  WSABASEERR         =  10000;

  WSAEINTR           =  (WSABASEERR + 4);
  WSAEBADF           =  (WSABASEERR + 9);
  WSAEACCES          =  (WSABASEERR + 13);
  WSAEFAULT          =  (WSABASEERR + 14);
  WSAEINVAL          =  (WSABASEERR + 22);
  WSAEMFILE          =  (WSABASEERR + 24);

  WSAEWOULDBLOCK     =  (WSABASEERR + 35);
  WSAEINPROGRESS     =  (WSABASEERR + 36);
  WSAEALREADY        =  (WSABASEERR + 37);
  WSAENOTSOCK        =  (WSABASEERR + 38);
  WSAEDESTADDRREQ    =  (WSABASEERR + 39);
  WSAEMSGSIZE        =  (WSABASEERR + 40);
  WSAEPROTOTYPE      =  (WSABASEERR + 41);
  WSAENOPROTOOPT     =  (WSABASEERR + 42);
  WSAEPROTONOSUPPORT =  (WSABASEERR + 43);
  WSAESOCKTNOSUPPORT =  (WSABASEERR + 44);
  WSAEOPNOTSUPP      =  (WSABASEERR + 45);
  WSAEPFNOSUPPORT    =  (WSABASEERR + 46);
  WSAEAFNOSUPPORT    =  (WSABASEERR + 47);
  WSAEADDRINUSE      =  (WSABASEERR + 48);
  WSAEADDRNOTAVAIL   =  (WSABASEERR + 49);
  WSAENETDOWN        =  (WSABASEERR + 50);
  WSAENETUNREACH     =  (WSABASEERR + 51);
  WSAENETRESET       =  (WSABASEERR + 52);
  WSAECONNABORTED    =  (WSABASEERR + 53);
  WSAECONNRESET      =  (WSABASEERR + 54);
  WSAENOBUFS         =  (WSABASEERR + 55);
  WSAEISCONN         =  (WSABASEERR + 56);
  WSAENOTCONN        =  (WSABASEERR + 57);
  WSAESHUTDOWN       =  (WSABASEERR + 58);
  WSAETOOMANYREFS    =  (WSABASEERR + 59);
  WSAETIMEDOUT       =  (WSABASEERR + 60);
  WSAECONNREFUSED    =  (WSABASEERR + 61);
  WSAELOOP           =  (WSABASEERR + 62);
  WSAENAMETOOLONG    =  (WSABASEERR + 63);
  WSAEHOSTDOWN       =  (WSABASEERR + 64);
  WSAEHOSTUNREACH    =  (WSABASEERR + 65);
  WSAENOTEMPTY       =  (WSABASEERR + 66);
  WSAEPROCLIM        =  (WSABASEERR + 67);
  WSAEUSERS          =  (WSABASEERR + 68);
  WSAEDQUOT          =  (WSABASEERR + 69);
  WSAESTALE          =  (WSABASEERR + 70);
  WSAEREMOTE         =  (WSABASEERR + 71);

  WSASYSNOTREADY         = (WSABASEERR + 91);
  WSAVERNOTSUPPORTED     = (WSABASEERR + 92);
  WSANOTINITIALISED      = (WSABASEERR + 93);
  WSAEDISCON             = (WSABASEERR + 101);
  WSAENOMORE             = (WSABASEERR + 102);
  WSAECANCELLED          = (WSABASEERR + 103);
  WSAEINVALIDPROCTABLE   = (WSABASEERR + 104);
  WSAEINVALIDPROVIDER    = (WSABASEERR + 105);
  WSAEPROVIDERFAILEDINIT = (WSABASEERR + 106);
  WSASYSCALLFAILURE      = (WSABASEERR + 107);
  WSASERVICE_NOT_FOUND   = (WSABASEERR + 108);
  WSATYPE_NOT_FOUND      = (WSABASEERR + 109);
  WSA_E_NO_MORE          = (WSABASEERR + 110);
  WSA_E_CANCELLED        = (WSABASEERR + 111);
  WSAEREFUSED            = (WSABASEERR + 112);

  WSAHOST_NOT_FOUND  =  (WSABASEERR + 1001);
  HOST_NOT_FOUND     =  WSAHOST_NOT_FOUND;

  WSATRY_AGAIN       =  (WSABASEERR + 1002);
  TRY_AGAIN          =  WSATRY_AGAIN;

  WSANO_RECOVERY     =  (WSABASEERR + 1003);
  NO_RECOVERY        =  WSANO_RECOVERY;

  WSANO_DATA         =  (WSABASEERR + 1004);
  NO_DATA            =  WSANO_DATA;

  WSANO_ADDRESS      =  WSANO_DATA;
  NO_ADDRESS         =  WSANO_ADDRESS;

  EWOULDBLOCK        =  WSAEWOULDBLOCK;
  EINPROGRESS        =  WSAEINPROGRESS;
  EALREADY           =  WSAEALREADY;
  ENOTSOCK           =  WSAENOTSOCK;
  EDESTADDRREQ       =  WSAEDESTADDRREQ;
  EMSGSIZE           =  WSAEMSGSIZE;
  EPROTOTYPE         =  WSAEPROTOTYPE;
  ENOPROTOOPT        =  WSAENOPROTOOPT;
  EPROTONOSUPPORT    =  WSAEPROTONOSUPPORT;
  ESOCKTNOSUPPORT    =  WSAESOCKTNOSUPPORT;
  EOPNOTSUPP         =  WSAEOPNOTSUPP;
  EPFNOSUPPORT       =  WSAEPFNOSUPPORT;
  EAFNOSUPPORT       =  WSAEAFNOSUPPORT;
  EADDRINUSE         =  WSAEADDRINUSE;
  EADDRNOTAVAIL      =  WSAEADDRNOTAVAIL;
  ENETDOWN           =  WSAENETDOWN;
  ENETUNREACH        =  WSAENETUNREACH;
  ENETRESET          =  WSAENETRESET;
  ECONNABORTED       =  WSAECONNABORTED;
  ECONNRESET         =  WSAECONNRESET;
  ENOBUFS            =  WSAENOBUFS;
  EISCONN            =  WSAEISCONN;
  ENOTCONN           =  WSAENOTCONN;
  ESHUTDOWN          =  WSAESHUTDOWN;
  ETOOMANYREFS       =  WSAETOOMANYREFS;
  ETIMEDOUT          =  WSAETIMEDOUT;
  ECONNREFUSED       =  WSAECONNREFUSED;
  ELOOP              =  WSAELOOP;
  ENAMETOOLONG       =  WSAENAMETOOLONG;
  EHOSTDOWN          =  WSAEHOSTDOWN;
  EHOSTUNREACH       =  WSAEHOSTUNREACH;
  ENOTEMPTY          =  WSAENOTEMPTY;
  EPROCLIM           =  WSAEPROCLIM;
  EUSERS             =  WSAEUSERS;
  EDQUOT             =  WSAEDQUOT;
  ESTALE             =  WSAESTALE;
  EREMOTE            =  WSAEREMOTE;

(* --- WINSOCK function prototypes --------------------------------------------------------------*)
TYPE
  FAccept                   = FUNCTION (s : TSocket; Addr : pSockAddr; AddrLen : pINTEGER) : TSocket; STDCALL;
  FBind                     = FUNCTION (s : TSocket; Addr : pSockAddr; NameLen : INTEGER) : INTEGER; STDCALL;
  FCloseSocket              = FUNCTION (s : TSocket) : INTEGER; STDCALL;
  FConnect                  = FUNCTION (s : TSocket; name : pSockAddr; NameLen : INTEGER) : INTEGER; STDCALL;
  FIoctlSocket              = FUNCTION (s : TSocket; cmd  : LONGINT;   argp    : p_Long) : INTEGER; STDCALL;
  FGetPeerName              = FUNCTION (s : TSocket; Name : pSockAddr; NameLen : pINTEGER) : INTEGER; STDCALL;
  FGetSockName              = FUNCTION (s : TSocket; Name : pSockAddr; NameLen : pINTEGER) : INTEGER; STDCALL;
  FGetSockOpt               = FUNCTION (s : TSocket; level, optname : WORD; optval : PChar; optlen : INTEGER) : INTEGER; STDCALL;
  FHtoNl                    = FUNCTION (hostlong  : u_long)  : u_long; STDCALL;
  FHtoNs                    = FUNCTION (hostshort : u_short) : u_short; STDCALL;
  FInet_Addr                = FUNCTION (cp : PChar) : u_long; STDCALL;
  FInet_NtoA                = FUNCTION (inaddr : TInAddr) : PChar; STDCALL;
  FListen                   = FUNCTION (s : TSocket; Backlog : INTEGER) : INTEGER; STDCALL;
  FNtoHl                    = FUNCTION (netlong  : u_long)  : u_long; STDCALL;
  FNtoHs                    = FUNCTION (netshort : u_short) : u_short; STDCALL;
  FRecv                     = FUNCTION (s : TSocket; buf : PChar; len, flags : INTEGER) : INTEGER; STDCALL;
  FRecvFrom                 = FUNCTION (s : TSocket; buf : PChar; len, flags : INTEGER; from : pSockAddr; FromLen : pINTEGER) : INTEGER; STDCALL;
  FSelect                   = FUNCTION (nfds : INTEGER; readfds, writefds, exceptfds : TFd_Set; timeout : timeval) : LONGINT; STDCALL;
  FSend                     = FUNCTION (s : TSocket; buf : PChar; len, flags : INTEGER) : INTEGER; STDCALL;
  FSendTo                   = FUNCTION (s : TSocket; buf : PChar; len, flags : INTEGER; addrto : pSockAddr; tolen : INTEGER) : INTEGER; STDCALL;
  FSetSockOpt               = FUNCTION (s : TSocket; level, optname : WORD; optval : PChar; optlen : INTEGER) : INTEGER; STDCALL;
  FShutDown                 = FUNCTION (s : TSocket; how : INTEGER) : INTEGER; STDCALL;
  FSocket                   = FUNCTION (af, struct, protocol : INTEGER) : TSocket; STDCALL;
  FGetHostByAddr            = FUNCTION (addr : PChar; len, struct : INTEGER) : pHostEnt; STDCALL;
  FGetHostByName            = FUNCTION (name : PChar) : pHostEnt; STDCALL;
  FGetServByPort            = FUNCTION (port : INTEGER; proto : PChar) : pServEnt; STDCALL;
  FGetServByName            = FUNCTION (name, proto : PChar) : PServEnt; STDCALL;
  FGetHostName              = FUNCTION (name : PChar; len : INTEGER): INTEGER; STDCALL;
  FGetProtoByNumber         = FUNCTION (proto : INTEGER) : PProtoEnt; STDCALL;
  FGetProtoByName           = FUNCTION (name : PChar) : pProtoEnt; STDCALL;
  FWsaStartup               = FUNCTION (wVersionRequired : WORD; lpWSData : pWsaData) : INTEGER; STDCALL;
  FWsaCleanup               = FUNCTION : INTEGER; STDCALL;
  FWsaSetLastError          = PROCEDURE (iError : INTEGER); STDCALL;
  FWsaGetLastError          = FUNCTION : INTEGER; STDCALL;
  FWsaIsBlocking            = FUNCTION : BOOL; STDCALL;
  FWSAUnhookBlockingHook    = FUNCTION : INTEGER; STDCALL;
  FWSASetBlockingHook       = FUNCTION (lpBlockFunc : TFarProc) : TFarProc; STDCALL;
  FWSACancelBlockingCall    = FUNCTION : INTEGER; STDCALL;
  FWSAAsyncGetServByName    = FUNCTION (HWindow : HWND; wMsg : u_int; name, proto, buf : PChar; buflen : INTEGER) : THandle; STDCALL;
  FWSAAsyncGetServByPort    = FUNCTION ( HWindow : HWND; wMsg, port : u_int; proto, buf : PChar; buflen : INTEGER) : THandle; STDCALL;
  FWSAAsyncGetProtoByName   = FUNCTION (HWindow : HWND; wMsg : u_int; name, buf : PChar; buflen : INTEGER) : THandle; STDCALL;
  FWSAAsyncGetProtoByNumber = FUNCTION (HWindow : HWND; wMsg : u_int; number : INTEGER;
                                        buf : PChar; buflen : INTEGER) : THandle; STDCALL;
  FWSAAsyncGetHostByName    = FUNCTION (HWindow : HWND; wMsg : u_int; name, buf : PChar; buflen : INTEGER) : THandle; STDCALL;
  FWSAAsyncGetHostByAddr    = FUNCTION (HWindow : HWND; wMsg : u_int; addr : PChar;
                                        len, struct : INTEGER;
                                        buf : PChar; buflen : INTEGER) : THandle; STDCALL;
  FWSACancelAsyncRequest    = FUNCTION (hAsyncTaskHandle : THandle) : INTEGER; STDCALL;
  FWSAAsyncSelect           = FUNCTION (s : TSocket; HWindow : HWND; wMsg : u_int;
                                        lEvent : LONGINT) : INTEGER; STDCALL;

(* --- WINSOCK Function Variables -------------------------------------------------------------- *)
VAR
  Accept                   : FAccept;
  Bind                     : FBind;
  CloseSocket              : FCloseSocket;
  Connect                  : FConnect;
  IoctlSocket              : FIoctlSocket;
  GetPeerName              : FGetPeerName;
  GetSockName              : FGetSockName;
  GetSockOpt               : FGetSockOpt;
  HtoNl                    : FHtoNl;
  HtoNs                    : FHtoNs;
  Inet_Addr                : FInet_Addr;
  Inet_NtoA                : FInet_NtoA;
  Listen                   : FListen;
  NtoHl                    : FNtoHl;
  NtoHs                    : FNtoHs;
  Recv                     : FRecv;
  RecvFrom                 : FRecvFrom;
  Select                   : FSelect;
  Send                     : FSend;
  SendTo                   : FSendTo;
  SetSockOpt               : FSetSockOpt;
  ShutDown                 : FShutDown;
  Socket                   : FSocket;
  GetHostByAddr            : FGetHostByAddr;
  GetHostByName            : FGetHostByName;
  GetServByPort            : FGetServByPort;
  GetServByName            : FGetServByName;
  GetHostName              : FGetHostName;
  GetProtoByNumber         : FGetProtoByNumber;
  GetProtoByName           : FGetProtoByName;
  WsaStartup               : FWsaStartup;
  WsaCleanup               : FWsaCleanup;
  WsaSetLastError          : FWsaSetLastError;
  WsaGetLastError          : FWsaGetLastError;
  WsaIsBlocking            : FWsaIsBlocking;
  WSAUnhookBlockingHook    : FWSAUnhookBlockingHook;
  WSASetBlockingHook       : FWSASetBlockingHook;
  WSACancelBlockingCall    : FWSACancelBlockingCall;
  WSAAsyncGetServByName    : FWSAAsyncGetServByName;
  WSAAsyncGetServByPort    : FWSAAsyncGetServByPort;
  WSAAsyncGetProtoByName   : FWSAAsyncGetProtoByName;
  WSAAsyncGetProtoByNumber : FWSAAsyncGetProtoByNumber;
  WSAAsyncGetHostByName    : FWSAAsyncGetHostByName;
  WSAAsyncGetHostByAddr    : FWSAAsyncGetHostByAddr;
  WSACancelAsyncRequest    : FWSACancelAsyncRequest;
  WSAAsyncSelect           : FWSAAsyncSelect;

(* --- WINSOCK Macros ---------------------------------------------------------------------- *)
FUNCTION WSAMakeSyncReply   (Buflen, Error : Word) : LongInt;
FUNCTION WSAMakeSelectReply (Event, Error : Word) : LongInt;
FUNCTION WSAGetAsyncBuflen  (Param : LongInt) : Word;
FUNCTION WSAGetAsyncError   (Param : LongInt) : Word;
FUNCTION WSAGetSelectEvent  (Param : LongInt) : Word;
FUNCTION WSAGetSelectError  (Param : LongInt) : Word;


(*
===============================================================================================
IMPLEMENTATION
===============================================================================================
*)

IMPLEMENTATION

VAR
  LibHandle : THandle;

PROCEDURE InitProcInstances;
BEGIN
  Accept                   := NIL;
  Bind                     := NIL;
  CloseSocket              := NIL;
  Connect                  := NIL;
  IoctlSocket              := NIL;
  GetPeerName              := NIL;
  GetSockName              := NIL;
  GetSockOpt               := NIL;
  HtoNl                    := NIL;
  HtoNs                    := NIL;
  Inet_Addr                := NIL;
  Inet_NtoA                := NIL;
  Listen                   := NIL;
  NtoHl                    := NIL;
  NtoHs                    := NIL;
  Recv                     := NIL;
  RecvFrom                 := NIL;
  Select                   := NIL;
  Send                     := NIL;
  SendTo                   := NIL;
  SetSockOpt               := NIL;
  ShutDown                 := NIL;
  Socket                   := NIL;
  GetHostByAddr            := NIL;
  GetHostByName            := NIL;
  GetServByPort            := NIL;
  GetServByName            := NIL;
  GetHostName              := NIL;
  GetProtoByNumber         := NIL;
  GetProtoByName           := NIL;
  WsaStartup               := NIL;
  WsaCleanup               := NIL;
  WsaSetLastError          := NIL;
  WsaGetLastError          := NIL;
  WsaIsBlocking            := NIL;
  WSAUnhookBlockingHook    := NIL;
  WSASetBlockingHook       := NIL;
  WSACancelBlockingCall    := NIL;
  WSAAsyncGetServByName    := NIL;
  WSAAsyncGetServByPort    := NIL;
  WSAAsyncGetProtoByName   := NIL;
  WSAAsyncGetProtoByNumber := NIL;
  WSAAsyncGetHostByName    := NIL;
  WSAAsyncGetHostByAddr    := NIL;
  WSACancelAsyncRequest    := NIL;
  WSAAsyncSelect           := NIL;
END;

PROCEDURE LoadWinsockDll;
VAR
  SEM : WORD;
BEGIN
  IF WinsockDllLoaded THEN EXIT;
  SEM := SetErrorMode (sem_NoOpenFileErrorBox);
  LibHandle := LoadLibrary ('WSOCK32.DLL');
  IF LibHandle = 0 THEN EXIT;
  SetErrorMode (SEM);

  @Accept                  := GetProcAddress (LibHandle, PChar (  1));
  @Bind                    := GetProcAddress (LibHandle, PChar (  2));
  @CloseSocket             := GetProcAddress (LibHandle, PChar (  3));
  @Connect                 := GetProcAddress (LibHandle, PChar (  4));
  @GetPeerName             := GetProcAddress (LibHandle, PChar (  5));
  @GetSockName             := GetProcAddress (LibHandle, PChar (  6));
  @GetSockOpt              := GetProcAddress (LibHandle, PChar (  7));
  @HtoNl                   := GetProcAddress (LibHandle, PChar (  8));
  @HtoNs                   := GetProcAddress (LibHandle, PChar (  9));
  @Inet_Addr               := GetProcAddress (LibHandle, PChar ( 10));
  @Inet_NtoA               := GetProcAddress (LibHandle, PChar ( 11));
  @IoctlSocket             := GetProcAddress (LibHandle, PChar ( 12));
  @Listen                  := GetProcAddress (LibHandle, PChar ( 13));
  @NtoHl                   := GetProcAddress (LibHandle, PChar ( 14));
  @NtoHs                   := GetProcAddress (LibHandle, PChar ( 15));
  @Recv                    := GetProcAddress (LibHandle, PChar ( 16));
  @RecvFrom                := GetProcAddress (LibHandle, PChar ( 17));
  @Select                  := GetProcAddress (LibHandle, PChar ( 18));
  @Send                    := GetProcAddress (LibHandle, PChar ( 19));
  @SendTo                  := GetProcAddress (LibHandle, PChar ( 20));
  @SetSockOpt              := GetProcAddress (LibHandle, PChar ( 21));
  @ShutDown                := GetProcAddress (LibHandle, PChar ( 22));
  @Socket                  := GetProcAddress (LibHandle, PChar ( 23));
  @GetHostByAddr           := GetProcAddress (LibHandle, PChar ( 51));
  @GetHostByName           := GetProcAddress (LibHandle, PChar ( 52));
  @GetProtoByName          := GetProcAddress (LibHandle, PChar ( 53));
  @GetProtoByNumber        := GetProcAddress (LibHandle, PChar ( 54));
  @GetServByName           := GetProcAddress (LibHandle, PChar ( 55));
  @GetServByPort           := GetProcAddress (LibHandle, PChar ( 56));
  @GetHostName             := GetProcAddress (LibHandle, PChar ( 57));
  @WSAAsyncSelect          := GetProcAddress (LibHandle, PChar (101));
  @WSAAsyncGetHostByAddr   := GetProcAddress (LibHandle, PChar (102));
  @WSAAsyncGetHostByName   := GetProcAddress (LibHandle, PChar (103));
  @WSAAsyncGetProtoByNumber:= GetProcAddress (LibHandle, PChar (104));
  @WSAAsyncGetProtoByName  := GetProcAddress (LibHandle, PChar (105));
  @WSAAsyncGetServByPort   := GetProcAddress (LibHandle, PChar (106));
  @WSAAsyncGetServByName   := GetProcAddress (LibHandle, PChar (107));
  @WSACancelAsyncRequest   := GetProcAddress (LibHandle, PChar (108));
  @WSASetBlockingHook      := GetProcAddress (LibHandle, PChar (109));
  @WSAUnhookBlockingHook   := GetProcAddress (LibHandle, PChar (110));
  @WsaGetLastError         := GetProcAddress (LibHandle, PChar (111));
  @WsaSetLastError         := GetProcAddress (LibHandle, PChar (112));
  @WSACancelBlockingCall   := GetProcAddress (LibHandle, PChar (113));
  @WsaIsBlocking           := GetProcAddress (LibHandle, PChar (114));
  @WsaStartup              := GetProcAddress (LibHandle, PChar (115));
  @WsaCleanup              := GetProcAddress (LibHandle, PChar (116));
END;


PROCEDURE UnloadWinsockDLL;
BEGIN
  IF LibHandle=0 THEN EXIT;
  FreeLibrary (LibHandle);
  InitProcInstances;
  LibHandle := 0;
END;


FUNCTION  WinsockDllLoaded : BOOLEAN;
BEGIN
  WinsockDllLoaded := LibHandle <> 0;
END;


(* --- Macro functions -------------------------------------------------------------------- *)

function WSAMakeSyncReply (Buflen, Error : Word) : LongInt;
begin
  WSAMakeSyncReply := MakeLong (Buflen, Error);
end;

function WSAMakeSelectReply (Event, Error : Word) : LongInt;
begin
  WSAMakeSelectReply := MakeLong (Event, Error);
end;

function WSAGetAsyncBuflen  (Param : LongInt) : Word;
begin
  WSAGetAsyncBuflen := LOWORD (Param);
end;

function WSAGetAsyncError  (Param : LongInt) : Word;
begin
  WSAGetAsyncError := HIWORD (Param);
end;

function WSAGetSelectEvent  (Param : LongInt) : Word;
begin
  WSAGetSelectEvent := LOWORD (Param);
end;

function WSAGetSelectError  (Param : LongInt) : Word;
begin
  WSAGetSelectError := HIWORD(Param);
end;


(* --- ErrorText function ------------------------------------------------------------------- *)

FUNCTION  ErrorText (Error : INTEGER) : STRING;
BEGIN
  CASE Error OF
    WSAEADDRINUSE       : ErrorText := 'Address already in use';
    WSAEADDRNOTAVAIL    : ErrorText := 'Cannot assign requested address';
    WSAEAFNOSUPPORT     : ErrorText := 'Address family not supported by protocol family';
    WSAEALREADY         : ErrorText := 'Operation already in progress';
    WSAECONNABORTED     : ErrorText := 'Software caused connection abort';
    WSAECONNREFUSED     : ErrorText := 'Connection refused';
    WSAECONNRESET       : ErrorText := 'Connection reset by peer';
    WSAEDESTADDRREQ     : ErrorText := 'Destination address required';
    WSAEFAULT           : ErrorText := 'Bad address';
    WSAEHOSTDOWN        : ErrorText := 'Host is down';
    WSAEHOSTUNREACH     : ErrorText := 'No route to host';
    WSAEINPROGRESS      : ErrorText := 'Operation now in progress';
    WSAEINTR            : ErrorText := 'Interrupted function call';
    WSAEINVAL           : ErrorText := 'Invalid argument';
    WSAEISCONN          : ErrorText := 'Socket is already connected';
    WSAEMFILE           : ErrorText := 'Too many open files';
    WSAEMSGSIZE         : ErrorText := 'Message too long';
    WSAENETDOWN         : ErrorText := 'Network is down';
    WSAENETRESET        : ErrorText := 'Network dropped connection on reset';
    WSAENETUNREACH      : ErrorText := 'Network is unreachable';
    WSAENOBUFS          : ErrorText := 'No buffer space available';
    WSAENOPROTOOPT      : ErrorText := 'Bad protocol option';
    WSAENOTCONN         : ErrorText := 'Socket is not connected';
    WSAENOTSOCK         : ErrorText := 'Socket operation on non-socket';
    WSAEOPNOTSUPP       : ErrorText := 'Operation not supported';
    WSAEPFNOSUPPORT     : ErrorText := 'Protocol family not supported';
    WSAEPROCLIM         : ErrorText := 'Too many processes';
    WSAEPROTONOSUPPORT  : ErrorText := 'Protocol not supported';
    WSAEPROTOTYPE       : ErrorText := 'Protocol wrong type for socket';
    WSAESHUTDOWN        : ErrorText := 'Cannot send after socket shutdown';
    WSAESOCKTNOSUPPORT  : ErrorText := 'Socket type not supported';
    WSAETIMEDOUT        : ErrorText := 'Connection timed out';
    WSAEWOULDBLOCK      : ErrorText := 'Operation would block';
    WSAHOST_NOT_FOUND   : ErrorText := 'Host not found';
    WSANOTINITIALISED   : ErrorText := 'Successful WSAStartup() not yet performed';
    WSANO_DATA          : ErrorText := 'Valid name, no data record of requested type';
    WSANO_RECOVERY      : ErrorText := 'This is a non-recoverable error';
    WSASYSNOTREADY      : ErrorText := 'Network subsystem is unavailable';
    WSATRY_AGAIN        : ErrorText := 'Non-authoritative host not found';
    WSAVERNOTSUPPORTED  : ErrorText := 'WINSOCK.DLL version out of range';
    WSAEDISCON              : ErrorText := 'WSAEDISCON';
    WSAENOMORE              : ErrorText := 'WSAENOMORE';
    WSAECANCELLED           : ErrorText := 'WSAECANCELLED';
    WSAEINVALIDPROCTABLE    : ErrorText := 'WSAEINVALIDPROCTABLE';
    WSAEINVALIDPROVIDER     : ErrorText := 'WSAEINVALIDPROVIDER';
    WSAEPROVIDERFAILEDINIT  : ErrorText := 'WSAEPROVIDERFAILEDINIT';
    WSASYSCALLFAILURE       : ErrorText := 'WSASYSCALLFAILURE';
    WSASERVICE_NOT_FOUND    : ErrorText := 'WSASERVICE_NOT_FOUND';
    WSATYPE_NOT_FOUND       : ErrorText := 'WSATYPE_NOT_FOUND';
    WSA_E_NO_MORE           : ErrorText := 'WSA_E_NO_MORE';
    WSA_E_CANCELLED         : ErrorText := 'WSA_E_CANCELLED';
    WSAEREFUSED             : ErrorText := 'WSAEREFUSED';
    ELSE                      ErrorText := 'Error #'+IntToStr (Error);
    END;
END;


FUNCTION  ErrorName (Error : INTEGER) : STRING;
BEGIN
  CASE Error OF
    WSAEADDRINUSE       : ErrorName := 'WSAEADDRINUSE';
    WSAEADDRNOTAVAIL    : ErrorName := 'WSAEADDRNOTAVAIL';
    WSAEAFNOSUPPORT     : ErrorName := 'WSAEAFNOSUPPORT';
    WSAEALREADY         : ErrorName := 'WSAEALREADY';
    WSAECONNABORTED     : ErrorName := 'WSAECONNABORTED';
    WSAECONNREFUSED     : ErrorName := 'WSAECONNREFUSED';
    WSAECONNRESET       : ErrorName := 'WSAECONNRESET';
    WSAEDESTADDRREQ     : ErrorName := 'WSAEDESTADDRREQ';
    WSAEFAULT           : ErrorName := 'WSAEFAULT';
    WSAEHOSTDOWN        : ErrorName := 'WSAEHOSTDOWN';
    WSAEHOSTUNREACH     : ErrorName := 'WSAEHOSTUNREACH';
    WSAEINPROGRESS      : ErrorName := 'WSAEINPROGRESS';
    WSAEINTR            : ErrorName := 'WSAEINTR';
    WSAEINVAL           : ErrorName := 'WSAEINVAL';
    WSAEISCONN          : ErrorName := 'WSAEISCONN';
    WSAEMFILE           : ErrorName := 'WSAEMFILE';
    WSAEMSGSIZE         : ErrorName := 'WSAEMSGSIZE';
    WSAENETDOWN         : ErrorName := 'WSAENETDOWN';
    WSAENETRESET        : ErrorName := 'WSAENETRESET';
    WSAENETUNREACH      : ErrorName := 'WSAENETUNREACH';
    WSAENOBUFS          : ErrorName := 'WSAENOBUFS';
    WSAENOPROTOOPT      : ErrorName := 'WSAENOPROTOOPT';
    WSAENOTCONN         : ErrorName := 'WSAENOTCONN';
    WSAENOTSOCK         : ErrorName := 'WSAENOTSOCK';
    WSAEOPNOTSUPP       : ErrorName := 'WSAEOPNOTSUPP';
    WSAEPFNOSUPPORT     : ErrorName := 'WSAEPFNOSUPPORT';
    WSAEPROCLIM         : ErrorName := 'WSAEPROCLIM';
    WSAEPROTONOSUPPORT  : ErrorName := 'WSAEPROTONOSUPPORT';
    WSAEPROTOTYPE       : ErrorName := 'WSAEPROTOTYPE';
    WSAESHUTDOWN        : ErrorName := 'WSAESHUTDOWN';
    WSAESOCKTNOSUPPORT  : ErrorName := 'WSAESOCKTNOSUPPORT';
    WSAETIMEDOUT        : ErrorName := 'WSAETIMEDOUT';
    WSAEWOULDBLOCK      : ErrorName := 'WSAEWOULDBLOCK';
    WSAHOST_NOT_FOUND   : ErrorName := 'WSAHOST_NOT_FOUND';
    WSANOTINITIALISED   : ErrorName := 'WSANOTINITIALISED';
    WSANO_DATA          : ErrorName := 'WSANO_DATA';
    WSANO_RECOVERY      : ErrorName := 'WSANO_RECOVERY';
    WSASYSNOTREADY      : ErrorName := 'WSASYSNOTREADY';
    WSATRY_AGAIN        : ErrorName := 'WSATRY_AGAIN';
    WSAVERNOTSUPPORTED  : ErrorName := 'WSAVERNOTSUPPORTED';
    WSAEDISCON              : ErrorName := 'WSAEDISCON';
    WSAENOMORE              : ErrorName := 'WSAENOMORE';
    WSAECANCELLED           : ErrorName := 'WSAECANCELLED';
    WSAEINVALIDPROCTABLE    : ErrorName := 'WSAEINVALIDPROCTABLE';
    WSAEINVALIDPROVIDER     : ErrorName := 'WSAEINVALIDPROVIDER';
    WSAEPROVIDERFAILEDINIT  : ErrorName := 'WSAEPROVIDERFAILEDINIT';
    WSASYSCALLFAILURE       : ErrorName := 'WSASYSCALLFAILURE';
    WSASERVICE_NOT_FOUND    : ErrorName := 'WSASERVICE_NOT_FOUND';
    WSATYPE_NOT_FOUND       : ErrorName := 'WSATYPE_NOT_FOUND';
    WSA_E_NO_MORE           : ErrorName := 'WSA_E_NO_MORE';
    WSA_E_CANCELLED         : ErrorName := 'WSA_E_CANCELLED';
    WSAEREFUSED             : ErrorName := 'WSAEREFUSED';
    ELSE                      ErrorName := '#'+IntToStr (Error);
    END;
END;


(*
===============================================================================================
INITIALIZATION
===============================================================================================
*)

INITIALIZATION
  LibHandle := 0;
  InitProcInstances;
END.

